//
//  JDLive.h
//  JDLive
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>
#import "JDLConfig.h"
#import "JDLiveDelegate.h"

/*!
   @class JDLive
   @abstract 京东云直播SDK-推流接口类
   @discussion 本类包含了所有推流相关的接口方法，包括初始化，音视频采集，音视频预览，视频处理，视频录制，推流控制，水印添加等功能
 */
@interface JDLive : NSObject

/*!
   @abstract   初始化配置类
   @param      config  配置类对象
   @result     JDLive实例
 */
- (instancetype)initWithConfig:(JDLConfig *)config;

/*!
   @abstract   推流回调代理
   @param      delegate  推流代理回调
   @discussion see JDLiveDelegate
 */
- (void)setLiveDelegate:(id<JDLiveDelegate>)delegate;

/*!
   @abstract   获取SDK版本号
   @result     版本号字符串
 */
- (NSString*)getSDKVersion;

/*!
 @abstract   查询摄像头是否支持预览
 @param      sessionPreset  分辨率
 @param      cameraPosition  摄像头
 @result     YES:支持
             NO:不支持
 */
+(BOOL)isCanSetSessionPreset:(NSString *)sessionPreset
          withCameraPosition:(AVCaptureDevicePosition)cameraPosition;

/*!
   @abstract   开始预览
   @param      view  传入预览,内部预览view将加到view上
 */
- (void)startPreview:(UIView *)view;

/*!
   @abstract   调整内部预览大小和外部预览view大小一致
 */
- (void)adjustPreviewSize;

/*!
   @abstract   停止预览
 */
- (void)stopPreview;

/*!
   @abstract   是否在录制
   @return     YES:正在录制
               NO:没有录制
 */
- (BOOL)isCaptureRunning;

/*!
   @abstract   切换摄像头
   @return     YES:成功切换摄像头
               NO:切换摄像头失败
 */
- (BOOL)switchCamera;

/*!
   @abstract   获取当前摄像头状态
   @return     AVCaptureDevicePosition:前置 后置
 */
- (AVCaptureDevicePosition)getCurrentDevicePosition;

/*!
    @abstract   获取当前相机
    @return     AVCaptureDevice:当前相机
    */
- (AVCaptureDevice*)getCurrentCaptureCamera;

/*!
   @abstract   闪光灯是否支持
   @return     YES:支持
               NO:不支持
 */
- (BOOL) isFlashLightSupported;

/*!
   @abstract   切换开关闪光灯
 */
- (void) toggleFlashLight;

/*!
   @abstract   设置视频缩放因子
   @param      factor  因子数
 */
- (void)setVideoZoomFactor:(CGFloat)factor;

/*!
   @abstract   获取视频缩放因子
   @return     因子数
 */
- (CGFloat)getVideoZoomFactor;

/*!
   @abstract   摄像头自动变焦
   @param      point  变焦位置
   @param      mode  变焦类型
   @return     YES:成功变焦
               NO:变焦失败
 */
- (BOOL)setCaptureFocusPoint:(CGPoint )point FocusMode:(AVCaptureFocusMode)mode;

/*!
   @abstract   摄像头自动曝光
   @param      point  曝光位置
   @return     YES:成功曝光
               NO:曝光失败
 */
- (BOOL)setCaptureExposureAtPoint:(CGPoint )point;

/*!
 @abstract   摄像头白平衡
 @return     YES:成功
             NO:失败
 */
- (BOOL)setCaptureWhiteBalance;

/*!
   @abstract   获取摄像头分辨率
   @return     分辨率
 */
- (CGSize)getCaptureDimensions;

/*!
   @abstract   获取预览试图大小
   @return     大小
 */
- (CGSize)getPreViewSize;

/*!
   @abstract   设置推流地址
   @param      url  地址
 */
- (void)setPushStreamUrl:(NSURL*)url;

/*!
   @abstract   获取推流地址
   @return     地址
 */
- (NSURL*)getPushStreamUrl;

/*!
   @abstract   开始推流
 */
- (void)startPushStream;

/*!
   @abstract   停止推流
 */
- (void)stopPushStream;

/*!
   @abstract   录屏数据传入接口
   @param      sample  CMSampleBufferRef
   @param      type    RPSampleBufferType
 */
- (void)processReplayKitSampleBuffer:(CMSampleBufferRef)sample withSampleType:(int)type;

/*!
   @abstract   获取推流状态
   @return     JDLStreamState
 */
- (JDLPushStreamState)getPushStreamState;

/*!
   @abstract   查询当前是否在推流 (建立连接中, 或连接中)
   @return     YES:推流中
               NO:推流中
 */
- (BOOL) isPushStreaming;

/*!
   @abstract    设置获取是否仅推音频数据，视频画面发送最后一帧
   @param       isAudioOnly YES:开启此功能 NO:不开启此功能
 */
- (void)setPushStreamAudioOnly:(BOOL)isAudioOnly;

/*!
   @abstract    获取是否仅推音频数据，视频画面发送最后一帧
   @return      YES:开启此功能
                NO:不开启此功能
 */
- (BOOL)getPushStreamAudioOnly;

/*!
   @abstract    开始背景图+音频推流
   @param       image 背景图
 */
- (void)startBGImagePushStream:(UIImage*)image;

/*!
 @abstract    开始UIView+音频推流
 @param       view 用于推流的UIView
 */
- (void)startUIViewPushStream:(UIView*)view;

/*!
   @abstract    恢复正常视频+音频推流
 */
- (void)resumeNormalPushStream;

/*!
   @abstract    返回当前推流模式
   @return      JDLPushStreamMode
 */
- (JDLPushStreamMode)getPushStreamMode;

/*!
   @abstract    开始悬浮窗口预览
   @param       floatView 传入悬浮窗
 */
- (void)startFloatingPreview:(UIView*)floatView;

/*!
   @abstract    关闭悬浮窗口预览
 */
- (void)closeFloatingPreview;

/*!
   @abstract    设置预览屏幕方向
   @param       orie  预览屏幕方向
 */
- (void) setVideoPreviewOrientation: (UIInterfaceOrientation) orie;

/*!
  @abstract    获取预览屏幕方向
  @return      UIInterfaceOrientation  预览屏幕方向
 */
- (UIInterfaceOrientation) getVideoPreviewOrientation;

/*!
   @abstract   设置推流屏幕方向
   @param      orie  推流屏幕方向
 */
- (void) setVideoPushStreamOrientation: (UIInterfaceOrientation) orie;

/*!
   @abstract   获取推流屏幕方向
   @return     UIInterfaceOrientation  推流屏幕方向
 */
- (UIInterfaceOrientation) getVideoPushStreamOrientation;

/*!
   @abstract   是否插入耳机
   @return     YES:插入了耳机
               NO:未插入耳机
 */
- (BOOL)isHeadsetPlugIn;

/*!
   @abstract   设置是否开启耳返
   @param      enable YES:开启此功能 NO:不开启此功能（默认）
 */
- (void)setAudioFoldbackEnable:(BOOL)enable;

/*!
   @abstract   获取是否开启耳返
   @return     YES:开启此功能
               NO:不开启此功能
 */
- (BOOL)getAudioFoldbackEnable;

/*!
   @abstract   设置是否立体声推流
   @param      enable YES:开启此功能 NO:不开启此功能（默认）
 */
- (void)setAudioStreamStereoEnable:(BOOL)enable;

/*!
   @abstract   获取是否立体声推流
   @return     YES:开启此功能
               NO:不开启此功能
 */
- (BOOL)getAudioStreamStereoEnable;

/*!
   @abstract   设置是否静音推流
   @param      mute YES:静音 NO:不静音（默认）
 */
- (void)setAudioStreamMute:(BOOL)mute;

/*!
   @abstract   获取是否静音推流
   @return     YES:静音
               NO:不静音
 */
- (BOOL)getAudioStreamMute;

/*!
   @abstract   设置输入音频设备类型
   @param      type 设备类型
 */
- (void)setInputAudioDeviceType:(JDLAudioDeviceType)type;

/*!
   @abstract   获取输入音频设备类型
   @return     JDLAudioDeviceType 设备类型
 */
- (JDLAudioDeviceType)getInputAudioDeviceType;

/*!
   @abstract   设置音频混响类型
   @param      type 0关闭（默认）1录音棚 2演唱会 3KTV 4小舞台
 */
- (void)setAudioReverbType:(int)type;

/*!
   @abstract   获取音频混响类型
   @return     type（int 0关闭（默认）1录音棚 2演唱会 3KTV 4小舞台）
 */
- (int)getAudioReverbType;

/*!
   @abstract   设置音频变声效果类型
   @param      type （int 0原声（默认）1大叔 2萝莉 3宏大 4机器人）
 */
- (void)setAudioEffectType:(int)type;

/*!
   @abstract   获取音频变声效果类型
   @return     type （int 0原声（默认）1大叔 2萝莉 3宏大 4机器人）
 */
- (int)getAudioEffectType;

/*!
   @abstract   设置音频自定义升降调效果类型
   @param      flag  int -3 ~ 3
 */
- (void)setAudioEffectCustomFlag:(int)flag;

/*!
   @abstract   获取音频自定义升降调效果类型
   @return     flag  int 0-7
 */
- (int)getAudioEffectCustomFlag;

/*!
   @abstract   设置噪声处理参数
   @param      param 0关闭（默认）1-4
 */
- (void)setAudioNoiseHandleParam:(int)param;

/*!
   @abstract   获取噪声处理参数
   @return     0关闭（默认）1-4
 */
- (int)getAudioNoiseHandleParam;

/*!
   @abstract   设置耳返回音量
   @param      vol  0-1.0
 */
- (void)setAudioFoldbackVolume:(CGFloat)vol;

/*!
   @abstract   获取耳返回音量
   @return     0-1.0
 */
- (CGFloat)getAudioFoldbackVolume;

/*!
   @abstract   设置音轨混合是否开启
   @param      enable YES:开启此功能 NO:不开启此功能
   @param      trackId  0麦克风  1背景音乐
 */
- (void)setAudioMixEnable:(BOOL)enable withTrackId:(int)trackId;

/*!
   @abstract   获取音轨混合是否开启
   @param      trackId  0麦克风  1背景音乐
   @return     YES:开启此功能
               NO:不开启此功能
 */
- (BOOL)isAudioMixEnableWithTrackId:(int)trackId;

/*!
   @abstract   设置混音音量
   @param      vol  0-2.0
   @param      trackId 0麦克风  1背景音乐
 */
- (void)setAudioMixVolume:(CGFloat)vol withTrackId:(int)trackId;

/*!
   @abstract   获取混音音量
   @param      trackId 0麦克风  1背景音乐
   @return     vol：  0-2.0
 */
- (CGFloat)getAudioMixVolumeWithTrackId:(int)trackId;

/*!
   @abstract   开始播放背景音乐
   @param      audioPath 本地音乐文件
 */
-(void)startBGMusic:(NSString*)audioPath;

/*!
   @abstract   停止播放背景音乐
   @param      completion 完成停止音乐的回调
 */
-(void)stopBGMusicWithComplete:(void (^)(void))completion;

/*!
   @abstract   暂停播放背景音乐
 */
-(void)pauseBGMusic;

/*!
   @abstract   恢复播放背景音乐
 */
-(void)resumeBGMusic;

/*!
   @abstract   设置播放完成回调
   @param      block 播放完成回调
 */
- (void)setBGMusicFinishBlock:(void (^)(void))block;

/*!
   @abstract   设置播放状态改变回调
   @param      block 状态改变回调
 */
- (void)setBGMusicStateChangeBlock:(void (^)(void))block;

/*!
   @abstract   seek到指定时间 (拖动进度条)
   @param      time 指定时间(单位,秒)
   @param      completion 回调
 */
- (void) seekBGMusicToTime:(CGFloat)time withComplete:(void (^)(BOOL))completion;

/*!
   @abstract   设置播放速率
   @param      playRate 调整范围 0.5 ~ 2.0，默认值 1.0
 */
- (void) setBGMusicPlayRate:(CGFloat)playRate;

/*!
   @abstract   获取播放速率
   @return     调整范围 0.5 ~ 2.0，默认值 1.0
 */
- (CGFloat) getBGMusicPlayRate;

/*!
   @abstract   设置背景音乐播放音量
   @param      volume 调整范围 0.0~1.0
 */
- (void) setBGMusicVolume:(CGFloat)volume;

/*!
   @abstract   获取背景音乐播放音量
   @return     volume 调整范围 0.0~1.0
 */
- (CGFloat) getBGMusicVolume;

/*!
   @abstract   设置背景音乐播放静音，推流还有音量
   @param      mute YES:静音 NO:不静音(默认)
 */
- (void) setBGMusicMute:(BOOL)mute;

/*!
   @abstract   获取背景音乐播放静音，推流还有音量
   @return     YES:静音
               NO:不静音(默认)
 */
- (BOOL) getBGMusicMute;

/*!
   @abstract   获取背景音乐播放状态
   @return     JDLBGMusicPlayerState 背景音乐播放状态
 */
- (JDLBGMusicPlayerState)getBGMusicState;

/*!
   @abstract   获取背景音乐总时长
   @return     总时长 (单位,秒)
 */
- (CGFloat)getBGMusicDuration;

/*!
   @abstract   获取背景音乐当前播放时间点
   @return     时间点(单位,秒)
 */
- (CGFloat)getBGMusicCurrentPlayTime;

/*!
   @abstract   设置预览图像水平反转
   @param      flip YES:左右反转 NO:正常（默认）
 */
- (void)setPreviewHFlip:(BOOL)flip;

/*!
   @abstract   获取预览图像水平反转
   @return     YES:左右反转 NO:正常（默认）
 */
- (BOOL)getPreviewHFlip;

/*!
   @abstract   设置推流图像水平反转
   @param      flip YES:左右反转 NO:正常（默认）
 */
- (void)setPushStreamHFlip:(BOOL)flip;

/*!
   @abstract   获取推流图像水平反转
   @return     YES:左右反转 NO:正常（默认）
 */
- (BOOL)getPushStreamHFlip;

/*!
   @abstract   设置美颜等级接口
   @param      level 0 to 5 default=0
 */
- (void)setVideoBeautyWhiteLevel:(int)level;

/*!
   @abstract   获取美颜等级接口
   @return     0 to 5 default=0
 */
- (int)getVideoBeautyWhiteLevel;

/*!
   @abstract   设置滤镜效果接口
   @param      index 0 to 5 default=0
 */
- (void)setVideoFilterEffectIndex:(int)index;

/*!
   @abstract   获取滤镜效果接口
   @return     0 to 5 default=0
 */
- (int)getVideoFilterEffectIndex;

/*!
   @abstract   设置滤镜效果接口
   @param      strFile 滤镜文件全路径 default = nil
 */
- (void)setVideoFilterEffectFile:(NSString*)strFile;

/*!
   @abstract   获取滤镜效果接口
   @return     strFile 滤镜文件全路径
 */
- (NSString*)getVideoFilterEffectFile;


/*!
   @abstract   开始录制视频
   @param      url 本地录制路径
   @return     YES:成功 NO:失败
 */
- (BOOL) startMediaRecord:(NSURL*) url;

/*!
   @abstract   停止录制视频
 */
- (void) stopMediaRecord;

/*!
   @abstract   获取录制视频时长
   @return     时长
 */
- (double) getMediaRecordDuration;

/*!
   @abstract   获取录制视频状态
   @return     JDLMediaRecordState  录制视频状态
 */
- (JDLMediaRecordState) getMediaRecordState;

/*!
   @abstract   从推流编码之前的图像中截取一张
   @param      completion 通过完成代码块获取到截图完成的图像
 */
- (void) snapshotFromStreamWithCompletion:(void (^)(UIImage*))completion;

/*!
   @abstract   添加图片水印 一个图片
   @param      image        水印图片
   @param      rect         水印位置0～1
   @param      rotation     旋转角（单位：弧度）
   @param      alpha        透明度
   @param      rotationMode 水印方向
   @return     水印图层号
 */
- (int)addWaterWithImage:(UIImage*)image rect:(CGRect)rect rotation:(CGFloat)rotation alpha:(CGFloat)alpha rotationMode:(JDLRotationMode)rotationMode;

/*!
   @abstract   添加图片水印 一组图片
   @param      imageList      水印图片数组
   @param      duration       动图总时长
   @param      rect           水印位置0～1
   @param      rotation       旋转角（单位：弧度）
   @param      alpha          透明度
   @param      rotationMode   水印方向
   @return     水印图层号
 */
- (int)addWaterWithImageList:(NSArray<UIImage*>*)imageList duration:(CGFloat)duration rect:(CGRect)rect rotation:(CGFloat)rotation alpha:(CGFloat)alpha rotationMode:(JDLRotationMode)rotationMode;

/*!
   @abstract   添加文字水印 一个图片
   @param      view           文字水印图片
   @param      rect           水印位置0～1
   @param      rotation       旋转角（单位：弧度）
   @param      alpha          透明度
   @param      rotationMode   水印方向
   @return     水印图层号
 */
- (int)addTextWithView:(UIView*)view rect:(CGRect)rect rotation:(CGFloat)rotation alpha:(CGFloat)alpha rotationMode:(JDLRotationMode)rotationMode;

/*!
   @abstract   添加时间戳水印
   @param      label        时间戳水印文本
   @param      rect         水印位置0～1
   @param      rotation     旋转角（单位：弧度）
   @param      alpha        透明度
   @param      rotationMode   水印方向
   @return     水印图层号
 */
- (int)addTimestampWithLabel:(UILabel*)label rect:(CGRect)rect rotation:(CGFloat)rotation alpha:(CGFloat)alpha rotationMode:(JDLRotationMode)rotationMode;


/*!
   @abstract   添加View作为图层
   @param      view         作为图层的view
   @param      rect         水印位置0～1
   @param      rotation     旋转角（单位：弧度）
   @param      alpha        透明度
   @param      rotationMode   水印方向
   @return     水印图层号
 */
- (int)addUIViewWithView:(UIView*)view rect:(CGRect)rect rotation:(CGFloat)rotation alpha:(CGFloat)alpha rotationMode:(JDLRotationMode)rotationMode;

/*!
   @abstract   去除图层
   @param      layerId   图层号
 */
- (void)removeLayerWithId:(int)layerId;

/*!
  处理图片
 @param      lookUpImagePath   外部滤镜文件路径
 @param      preImage   原始图像
 @return     处理后图像
*/
+ (UIImage *)processEffectImageWithLookUpImagePath:(NSString *)lookUpImagePath preImage:(UIImage *)preImage;

/*!
  @abstract  发送用户自定义数据（通过SEI传输）
  @param     message  自定义消息 长度小于4KB
*/
-(void)sendUserCustomMessage:(NSString*)message;

/*!
  @abstract  加入一路混合视频流
  @param     index  当前流索引
*/
- (void)AddMixVideoStreamWithIndex:(int)index;

/*!
  @abstract  加入一路混合音频流
  @param     index  当前流索引
*/
- (void)AddMixAudioStreamWithIndex:(int)index;

/*!
  @abstract  移除一路混合视频流
  @param     index  当前流索引
*/
- (void)removeVideoStreamWithIndex:(int)index;

/*!
  @abstract  移除一路混合音频流
  @param     index  当前流索引
*/
- (void)removeAudioStreamWithIndex:(int)index;

/*!
  @abstract  输入混合流视频流数据
  @param     sampleBuffer  视频帧数据
  @param     index  当前流索引
*/
- (void)inputMixVideoSampleBuffer:(CMSampleBufferRef)sampleBuffer streamIndex:(int)index;

/*!
  @abstract  输入混合流音频数据
  @param     sampleBuffer  音频帧数据
  @param     index  当前流索引
*/
- (void)inputMixAudioSampleBuffer:(CMSampleBufferRef)sampleBuffer streamIndex:(int)index;

/*!
   @abstract  输出混合前视频数据
   @param     beforeMixVideoOutputSampleBufferCallback  回调函数
 */
-(void)setBeforeMixVideoOutputSampleBufferCallback:(void (^)(CMSampleBufferRef))beforeMixVideoOutputSampleBufferCallback;

/*!
   @abstract  输出混合前音频数据
   @param     beforeMixAudioOutputSampleBufferCallback  回调函数
 */
-(void)setBeforeMixAudioOutputSampleBufferCallback:(void (^)(CMSampleBufferRef))beforeMixAudioOutputSampleBufferCallback;

@end



