//
//  JDCloudPlay.h
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "JDCloudOptionModel.h"
#import "JDCloudPlayerDefine.h"
#import "JDCloudPlayerDelegate.h"

/*!
   @class JDCloudPlayer
   @abstract 京东云播放器SDK-播放接口类
   @discussion 本类包含了所有播放相关的接口方法，包括初始化，播放，暂停，gif等等功能
 */
@interface JDCloudPlayer : NSObject

/*!
   @abstract   播放回调代理
   @discussion see JDCloudPlayerDelegate
 */
@property (nonatomic, weak) id<JDCloudPlayerDelegate> delegate;

/*!
   @abstract 视频播放页面
 */
@property (nonatomic, readonly) UIView *playerView;

/*!
   @abstract 视频播放时间点

 */
@property (nonatomic) NSTimeInterval currentPlayerTime;

/*!
   @abstract 视频时长
 */
@property (nonatomic, readonly) NSTimeInterval duration;

/*!
   @abstract 视频可播放时长
 */
@property (nonatomic, readonly) NSTimeInterval playableDuration;

/*!
   @abstract 视频当前缓冲进度
 */
@property (nonatomic, readonly) NSInteger bufferingProgress;

/*!
   @abstract 是否开启超分增强
 */
@property (nonatomic, assign) BOOL isSREnable;

/*!
   @abstract 循环播放次数
 */
@property (nonatomic, assign) NSInteger loop;

/*!
   @abstract 播放是否准备好
 */
@property (nonatomic, readonly) BOOL isPreparedToPlay;

/*!
   @abstract 视频宽高尺寸
 */
@property (nonatomic, readonly) CGSize naturalSize;

/*!
   @abstract 设置是否自动播放
 */
@property(nonatomic, assign) BOOL shouldAutoplay;

/*!
   @abstract 播放倍速
 */
@property (nonatomic) float playRate;

/*!
   @abstract 播放音量 范围为0.0 - 1.0 默认为 1.0
 */
@property (nonatomic) float playVolume;

/*!
   @abstract 是否镜像播放
 */
@property (nonatomic, assign) BOOL isMirrorPlay;

/*!
   @abstract 是否纯音播放
 */
@property (nonatomic, assign) BOOL isOnlyAudio;

/*!
   @abstract 是否后台时播放
 */
@property (nonatomic, assign) BOOL isPlayOnBackGroud;

/*!
   @abstract 是否后台时暂停
 */
@property (nonatomic, assign) BOOL isPauseInBackground;

/*!
   @abstract 是否静音播放
 */
@property (nonatomic, assign) BOOL isMutePlaye;

/*!
   @abstract 是否隐藏视频
 */
@property (nonatomic, assign) BOOL isHidenPlayer;

/*!
   @abstract 获取已经加载的视频长度，或者说对于网络视频来说已经下载的视频时长
 */
@property (nonatomic, readonly, assign) NSTimeInterval loadedTime;

/*!
   @abstract 视频的状态
   @discussion see JDCloudPlayerState
 */
@property (nonatomic, readonly) JDCloudPlayerState playerState;

/*!
   @abstract 下载的状态
   @discussion see JDCloudLoadState
 */
@property (nonatomic, readonly) JDCloudLoadState loadState;

/*!
   @abstract 设置视频缩放模式
   @discussion see JDCloudScalingMode
 */
@property(nonatomic, assign) JDCloudScalingMode scalingMode;

/*!
   @abstract 视频原始数据的回调
   @discussion see JDCloudPlayerVideoRawDataBlock
 */
@property (nonatomic, copy) JDCloudPlayerVideoRawDataBlock videoRawDataBlock;

/*!
   @abstract 音频原始数据的回调
   @discussion see JDCloudPlayerAudioRawDataBlock
 */
@property (nonatomic, copy) JDCloudPlayerAudioRawDataBlock audioRawDataBlock;

/*!
  @abstract 音频原始数据的回调
  @discussion see audioPcmDataBlock
*/
@property (nonatomic, copy) JDCloudPlayerAudioPcmDataBlock audioPcmDataBlock;

/*!
   @abstract 视频旋转方向
   @discussion see JDCloudRenderRotate
 */
@property (nonatomic) JDCloudRenderRotate videoRotation;

/*!
 @abstract 设置直播视频缓存时间，以秒为单位
 @discussion see maxLiveBufferTime
 */
@property(nonatomic) NSInteger maxLiveBufferTime;

/*!
 @abstract 是否开启网络自适应，默认为关闭
 @discussion see adaptivePlayback
 */
@property(nonatomic, assign) BOOL adaptivePlayback;

/*!
 @abstract 是否关闭播放器缓冲，NO为开启缓冲，默认开启，YES为关闭
 @discussion see disableMemoryCache
 */
@property(nonatomic, assign) BOOL disableMemoryCache;

/*!
   @abstract 准备播放，调用后，不必再调用start
 */
- (void)prepareToPlay;

/*!
   @abstract 播放，相对于暂停，必须在调用prepareToPlay后，才可调用。
 */
- (void)start;

/*!
   @abstract 暂停
 */
- (void)pause;

/*!
   @abstract 暂停后恢复播放
 */
- (void)resume;

/*!
   @abstract 停止
 */
- (void)stop;

/*!
   @abstract 是否在播放
   @return  YES 在播放 NO 没有播放
 */
- (BOOL)isPlaying;

/*!
   @abstract 重新播放上一次url地址视频
 */
- (void)replay;

/*!
   @abstract 关闭播放器
 */
- (void)shutdown;

/*!
   @abstract 截图
   @return 截取的图
 */
- (UIImage *)thumbnailImageAtCurrentTime;

/*!
   @abstract 修改播放器frame
   @param frame  播放器大小
 */
- (void)resetPlayerViewFrame:(CGRect)frame;

/*!
   @abstract 跳转到指定位置进行播放
   @param time 指定位置的时间
 */
- (void)seekToTime:(NSTimeInterval)time;

/*!
   @abstract 根据本地视频URL初始化JDCloudPlay
   @param url  本地视频地址
   @param option 设置播放器配置选项，可以传nil，传nil的话，播放器采用默认配置选项
 */
- (void)setLocalVideoWithURL:(NSURL *)url withPlayerOption:(JDCloudOptionModel *)option;

/*!
   @abstract 根据本地视频URL初始化JDCloudPlay
   @param localVideoString  本地视频地址字符串
   @param option 设置播放器配置选项，可以传nil，传nil的话，播放器采用默认配置选项
 */
- (void)setLocalVideoWithString:(NSString *)localVideoString withPlayerOption:(JDCloudOptionModel *)option;

/*!
   @abstract 根据播放视频URL初始化JDCloudPlay 第一次播放
   @param url  视频地址
   @param option 设置播放器配置选项，可以传nil，传nil的话，播放器采用默认配置选项
 */
- (void)setVideoWithURL:(NSURL *)url withPlayerOption:(JDCloudOptionModel *)option;

/*!
   @abstract 根据新的播放视频URL与option重设播放器 重设地址
   @param url  视频地址
   @param option 设置播放器配置选项，可以传nil，传nil的话，播放器采用默认配置选项
 */
- (void)resetWithUrl:(NSURL *)url withPlayerOption:(JDCloudOptionModel *)option;

@end

