//
//  JDMediaEdit.h
//  JDShortvideo
//
//  Created by jdmt on 2019/4/1.
//  Copyright © 2019 com.jd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "JDMEDefines.h"
#import "JDSTimeLineItem.h"
#import "JDMEOutputConfig.h"
#import "JDMediaEditDelegate.h"

#import "JDSMovieTimeLineItem.h"
#import "JDSBGMTimeLineItem.h"
//#import "JDSCaptionTimeLineItem.h"
//#import "JDSStickerTimeLineItem.h"
#import "JDSVoiceTimeLineItem.h"
#import "JDSVideoFilterTimeLineItem.h"
#import "JDSAudioFilterTimeLineItem.h"
#import "JDSLayerTimeLineItem.h"

NS_ASSUME_NONNULL_BEGIN

/*!
 @class JDMediaEdit
 @abstract 京东云编辑SDK-视频编辑主类
 @discussion 本类包含了所有视频编辑预览和合成功能
 */
@interface JDMediaEdit : NSObject

/*!
 @abstract  初始化
 @param previewConfig 预览配置
 @param delegate 回调代理
 @return 返回编辑对象
 @discussion see JDMEPreviewConfig JDMediaEditDelegate
 */
- (instancetype)initWithPreviewConfig:(JDMEPreviewConfig*)previewConfig
                             delegate:(id<JDMediaEditDelegate>)delegate;

/*!
 @abstract   返回预览配置对象
 @result     返回预览配置对象
 @discussion see JDMEPreviewConfig
 */
- (JDMEPreviewConfig*)getPreviewConfig;

/*!
 @abstract   获取item层
 @param      type  时间线类型
 @result     该类型的item数组
 */
- (NSArray<JDSTimeLineItem*>*)getItemArrayByType:(JDSTimeLineItemType)type;

/*!
 @abstract   获取item层
 @param      time  时间
 @param      type  item类型，可以组合
 @result     返回的item数组
 */
- (NSArray<JDSTimeLineItem*>*)getItemArrayAtTime:(CMTime)time ofType:(JDSTimeLineItemType)type;

/*!
 @abstract   获取总时长
 @result     总时长
 */
- (CMTime)getTotalDuration;

/*!
 @abstract   在尾部添加时间线movieArray，异步调用
 @param      movieArray  要添加的movieArray；元素类型为PHAsset或者UIImage
 @param      completion  完成回调，返回YES表示成功，返回NO表示失败
 */
- (void)appendMovieArray:(NSArray*)movieArray completion:(void(^)(BOOL))completion;

/*!
 @abstract   在一个item之前插入一个时间线item
 @param      item  要插入的item
 @param      curItem  参考的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)insertMovieTimeLineItem:(JDSMovieTimeLineItem*)item beforeItem:(JDSMovieTimeLineItem*)curItem;

/*!
 @abstract   在一个item之后插入一个时间线item
 @param      item  要插入的item
 @param      curItem  参考的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)insertMovieTimeLineItem:(JDSMovieTimeLineItem*)item afterItem:(JDSMovieTimeLineItem*)curItem;

/*!
 @abstract   在尾部添加时间线item
 @param      item  要添加的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)appendMovieTimeLineItem:(JDSMovieTimeLineItem*)item;

/*!
 @abstract   删除一个时间线item
 @param      item  要删除的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)removeMovieTimeLineItem:(JDSMovieTimeLineItem *)item;

/*!
 @abstract   移动一个item到另一个item之前
 @param      item  要移动的item
 @param      curItem  参考的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)moveMovieTimeLineItem:(JDSMovieTimeLineItem*)item beforeItem:(JDSMovieTimeLineItem*)curItem;

/*!
 @abstract   移动一个item到另一个item之前
 @param      item  要移动的item
 @param      curItem  参考的item
 @result     成功返回YES，失败返回NO
 */
- (BOOL)moveMovieTimeLineItem:(JDSMovieTimeLineItem*)item afterItem:(JDSMovieTimeLineItem*)curItem;

/*!
 @abstract   分割一个movie item
 @param      time  分割时间点
 */
- (BOOL)splitMovieItemAtTime:(CMTime)time;

/*!
 @abstract   复制一个movie item，并放在后面
 @param      item  时间线item
 */
- (BOOL)copyMovieTimeLineItem:(JDSMovieTimeLineItem*)item;

/*!
 @abstract   设置视频裁剪的起始时间和终止时间
 @param      item  时间线item
 @param      startTime 起始时间，变速后
 @param      endTime 起始时间，变速后
 @result     成功返回YES，失败返回NO
 */
- (BOOL)clipMovieTimeLineItem:(JDSMovieTimeLineItem*)item withStartTime:(CMTime)startTime endTime:(CMTime)endTime;

/*!
 @abstract   设置播放速率
 @param      item  时间线item
 @param      rate  播放速率
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withRate:(double)rate;

/*!
 @abstract   设置滤镜名称
 @param      item  时间线item
 @param      filterName  滤镜名称
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withVideoFilterName:(NSString*)filterName;

/*!
 @abstract   设置旋转模式
 @param      item  时间线item
 @param      rotationMode  旋转模式GPUImageRotationMode
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withRotationMode:(int)rotationMode;

/*!
 @abstract   设置是否静音
 @param      item  时间线item
 @param      mute  是否静音
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withMute:(BOOL)mute;

/*!
 @abstract   设置音量
 @param      item  时间线item
 @param      volume  音量
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withVolume:(double)volume;


/*!
 @abstract   设置是否倒放
 @param      item  时间线item
 @param      reverse 是否倒放
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItem:(JDSMovieTimeLineItem*)item withReverse:(BOOL)reverse;


/*!
 @abstract   在尾部添加时间线itemArray
 @param      itemArray  要添加的itemArray
 @result     成功返回YES，失败返回NO
 */
- (BOOL)appendMovieTimeLineItemArray:(NSArray<JDSMovieTimeLineItem*>*)itemArray;

/*!
 @abstract   移除时间线item数组
 @param      itemArray  要添加的itemArray
 @result     成功返回YES，失败返回NO
 */
- (BOOL)removeMovieTimeLineItemArray:(NSArray<JDSMovieTimeLineItem*>*)itemArray;

/*!
 @abstract   设置播放速率
 @param      itemArray  时间线itemArray
 @param      rate  速率
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setMovieTimeLineItemArray:(NSArray<JDSMovieTimeLineItem*>*)itemArray withRate:(double)rate;


/*!
 @abstract  设置预览窗口
 @param view 预览窗口
 */
- (void)startPreview:(UIView *)view;

/*!
 @abstract  暂停预览
 */
- (void)pausePreview;

/*!
 @abstract  播放预览
 */
- (void)playPreview;

/*!
 @abstract  停止预览
 @discussion 重新开始预览，需要再调用startPreview
 */
- (void)stopPreview;

/*!
 @abstract  返回当前预览时间
 @return 返回当前预览时间点CMTime
 */
- (CMTime)getCurrentPreviewTime;

/*!
 @abstract  返回当前预览状态
 @return 返回当前预览状态 JDMEPreviewState
  @discussion see JDMEPreviewState
 */
- (JDMEPreviewState)getCurrentPreviewState;

/*!
 @abstract  预览seek
 @param time seek时间点
 */
- (void)seekToTime:(CMTime)time;

/*!
 @abstract  预览seek
 @param time seek时间点
 @param completionHandler 完成回调
 */
- (void)seekToTime:(CMTime)time completionHandler:(void (^)(BOOL finished))completionHandler;

/*!
 @abstract  开始合成
 @param config 合成输出配置对象
 */
-(void)startExportWithConfig:(JDMEOutputConfig*)config;


/*!
 @abstract  根据RGB颜色查找图像,将预览图像生成新的效果图像
 @param lookUpImagePath RGB颜色查找图像路径
 @param preImage 预览图像
 @return 返回效果图像
 */
+ (UIImage *)getEffectImageWithLookUpImagePath:(NSString *)lookUpImagePath
                                      preImage:(UIImage *)preImage;


/*!
 @abstract   获取所有背景音乐数组
 @result     返回所有背景音乐数组
 */
- (NSArray *)getBGMTimeLineItemArray;
/*!
 @abstract   获取某个时间点的背景音乐数组
 @param      time  时间点
 @result     返回某个时间点的背景音乐数组
 */
- (NSArray *)getBGMTimeLineItemArrayAtTime:(CMTime)time;
/*!
 @abstract   添加背景音乐
 @param      timeLineItem  时间线timeLineItem
 @result     成功返回YES，失败返回NO
 */
- (BOOL)addBGMTimeLineItem:(JDSBGMTimeLineItem *)timeLineItem;
/*!
 @abstract   移除背景音乐
 @param      timeLineItem  时间线timeLineItem
 @result     成功返回YES，失败返回NO
 */
- (BOOL)removeBGMTimeLineItem:(JDSBGMTimeLineItem *)timeLineItem;
/*!
 @abstract   修改背景音乐在时间线上的时间范围与背景音乐相对于自己的开始与播放时长
 @param      timeLineItem  时间线timeLineItem
 @param      timeRangeInWhole  景音乐在时间线上的时间范围
 @param      timeRangeInAsset  背景音乐相对于自己的开始与播放时长
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setBGMTimeLineItem:(JDSBGMTimeLineItem *)timeLineItem
      withTimeRangeInWhole:(CMTimeRange)timeRangeInWhole
      withTimeRangeInAsset:(CMTimeRange)timeRangeInAsset;

/*!
 @abstract   设置背景音乐播放速率
 @param      item  时间线item
 @param      rate  背景音乐播放速率
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setBGMTimeLineItem:(JDSBGMTimeLineItem*)item withRate:(double)rate;

/*!
 @abstract   设置背景音乐是否静音
 @param      timeLineItem  时间线item
 @param      mute  是否静音 YES 为静音 NO 为正常音量播放
 */
- (void)setBGMTimeLineItem:(JDSBGMTimeLineItem *)timeLineItem
                  WithMute:(BOOL)mute;

/*!
 @abstract   设置背景音乐音量
 @param      timeLineItem  时间线item
 @param      volume  背景音乐音量
 */
- (void)setBGMTimeLineItem:(JDSBGMTimeLineItem *)timeLineItem
                WithVolume:(double)volume;

/*!
 @abstract   获取所有水印字幕时间线Item数组
 @result     返回所有水印字幕时间线Item数组
 */
- (NSArray *)getLayerTimeLineItemArray;
/*!
 @abstract   获取某个时间点的水印字幕时间线Item数组
 @param      time  时间点
 @result     返回某个时间点的水印字幕时间线Item数组
 */
- (NSArray *)getLayerTimeLineItemArrayAtTime:(CMTime)time;
/*!
 @abstract   添加水印字幕时间线Item
 @param      timeLineItem  时间线 timeLineItem
 @result     成功返回YES，失败返回NO
 */
- (BOOL)addLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem;
/*!
 @abstract   移除水印字幕时间线Item
 @param      timeLineItem  时间线timeLineItem
 @result     成功返回YES，失败返回NO
 */
- (BOOL)removeLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem;
/*!
 @abstract   修改水印字幕时间线Item在时间线上的时间范围
 @param      timeLineItem  时间线timeLineItem
 @param      timeRangeInWhole  水印字幕时间线Item在时间线上的时间范围
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem
        withTimeRangeInWhole:(CMTimeRange)timeRangeInWhole;

/*!
 @abstract   修改水印字幕是否可编辑
 @param      timeLineItem  时间线timeLineItem
 @param      canEdit  时间线timeLineItem是否可编辑
 @result     成功返回YES，失败返回NO
 */
- (BOOL)setLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem
                 withCanEdit:(BOOL)canEdit;

/*!
 @abstract   开始编辑水印字幕
 @param      timeLineItem  开始编辑的时间线timeLineItem
 */
- (void)beginEditLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem;
/*!
 @abstract   结束编辑水印字幕
 @param      timeLineItem  结束编辑的时间线timeLineItem
 */
- (void)endEditLayerTimeLineItem:(JDSLayerTimeLineItem *)timeLineItem;


@end

NS_ASSUME_NONNULL_END
